function demoGridKey
% make a rectangular grid of square patches
% set the keypressed  and buttondown functions so that 
%        a mouse click changes color from unselected to selected
%        a shifted mouse click unselects
%        arrow keys move to neighboring cells and select

%% set parameters
Nx=5;
Ny=6;
selectedColor=[1, 0, 0];
unselectedColor=[0, 0, 1];
sFactor=0.9;      % scale factor to make squares smaller and show gaps

hAx=gca;
setappdata(gcf,'selectedColor',selectedColor);
setappdata(gcf,'unselectedColor',unselectedColor);
setappdata(gcf,'SelectMode','select');

%% make grid of sqaire patches
clf
Xp=[0, 1, 1, 0]*sFactor;
Yp=[0, 0, 1, 1]*sFactor;
for iy=1:Ny
    for ix=1:Nx
        hp(ix, iy)=patch(Xp+ix-1, Yp+iy-1,unselectedColor);
    end
end
axis([0, Nx, 0, Ny]);
axis equal
axis off

%% set buttondown functions
for iy=1:Ny
    for ix=1:Nx
        hp(ix, iy).ButtonDownFcn={@myButtonDownFcn, hp, ix, iy};
    end
end

%% set keypress function
set(gcf,'KeyPressFcn',{@myProcessKey, hp});

%% mouse and keyboard functions

function myButtonDownFcn(hThis,eventdata,hp, ix, iy)
    setappdata(gcf,'CurrentPatch',[ix,iy]);
    selectedColor=getappdata(gcf,'selectedColor');
    unselectedColor=getappdata(gcf,'unselectedColor'); 
    currentColor=hThis.FaceColor;
    
    % if patch has selectedColor, then go to unselect mode
    %     otherwise select and go to select mode
    
    set(hp,'LineWidth',0.5);
    set(hp(ix,iy),'LineWidth',2);
    if currentColor==selectedColor
        hThis.FaceColor=unselectedColor;
        setappdata(gcf,'SelectMode','unselect');
    else
        hThis.FaceColor=selectedColor;
        setappdata(gcf,'SelectMode','select');
    end
    
 
    function  myProcessKey(hFig,event, hp)
        %% collect information
        patchxy=getappdata(gcf,'CurrentPatch');
        selectedColor=getappdata(gcf,'selectedColor');
        unselectedColor=getappdata(gcf,'unselectedColor');
        ix=patchxy(1);
        iy=patchxy(2);
        [Nx, Ny]=size(hp);      
        
        %% process key to find new position
        switch event.Key
            case 'uparrow'
                newx=ix;
                newy=iy+1;
            case 'downarrow'
                newx=ix;
                newy=iy-1;
            case 'leftarrow'
                newx=ix-1;
                newy=iy;
            case 'rightarrow'
                newx=ix+1;
                newy=iy;
        end
        
        %% check boundaries
        if newx<1
            newx=1;
        elseif newx > Nx
            newx=Nx;
        elseif newy > Ny
            newy=Ny;
        elseif newy <1
            newy=1;
        end
        
        %% highlight new square
        set(hp,'LineWidth',0.5);
        set(hp(newx,newy),'LineWidth',2);
        
        %% change color as appropriate to SelectMode
        selectmode=getappdata(gcf,'SelectMode');
        switch selectmode
            case 'select'
                hp(newx,newy).FaceColor=selectedColor;
            case 'unselect'
                hp(newx,newy).FaceColor=unselectedColor;
        end
        
        %% set new square as current patch
        setappdata(gcf,'CurrentPatch',[newx, newy]);
        
   
    
    
    
    
    
    
    
    